<?php
/*
* Class to convert the XML content into php variables and to insert the content into
* corresponding datebase tables.
*
* TODO:
*   implement "what about" - see below
*
* Software specification
*
* - check existing data first to avoid DB updates / inserts if not needed
* - use mass insert / update
*
*** player inserts
* - insert players using their ogame_playerid as unique identifier to detect renamings
* - own player has no ogame_playerid at the galaxyview
* - own playerid will be known if another tool user updates a system with this own user
* - when no ogame_playerid is provided, check usertable where own playerid is stored if known
* - if usertable has no playerid; detection of existing player can only work with playername
*
* - what about: player A without playerid inserted, renames to B (second player at DB)
*               B is updated with playerid from different user - A will still remain and no automatic merge possible
*               except there is a test on that provided coordinates with the data that was there before the update
*               In case the player there had no ogame-playerid too -> rename
*               For statistics this will work if the home planet is used and populated
*
*               Player A with playerid inserted by different user
*               Player A renames himself to B and updates himself without playerid = new user
*               B is updated with playerid once again ... what will happen to the two entries now?!
*               currently we will have two B players, one with playerid and one without - try to merge them
*
*/
require_once "error_object.inc.php";
require_once "parser_global.inc.php";
require_once "parser_activities.inc.php";

class Parser extends XMLParserGlobal{

	private $galaxytable = "";

	private $alliance_names_2_ids = array();
	private $player_names_2_ids   = array();

	private $collected_playernames = array();
	private $collected_allynames = array();

	private $activity_parser = null;

	function __construct($dbcon,$galaxytable,$allytable,$playertable,$utablename,$player_activitytable,$universe) {

		$this->xml_schema = "xml_schema/galaxyview.xsd";
		//  call super constructor
		$result = parent::__construct($dbcon,$allytable,$playertable,$utablename,$universe);
		if ($result === false) {
			return false;
		}
		if (trim($galaxytable) == "") return false;

		// local setup
		$this->galaxytable = $galaxytable;

		// create object to track player activities
		$this->activity_parser = new ActivityParser($dbcon,$player_activitytable,$utablename,$universe);
		if ($this->activity_parser === false) return false;

		return $this;
	}

	public function insert_data($xml_content) {
		$xdoc = $this->validate_header($xml_content,XMLParserGlobal::content_type_galaxyview);
		if ($xdoc === false) {
			return false;
		}

		$galaxyviews_items = $xdoc->getElementsByTagName("galaxyview"); // at least one
		$galaxyview_data = array();
		foreach($galaxyviews_items as $galaxyviews_item) {
			// extract information from XML file
			$system_data = $this->get_system_data($galaxyviews_item);
			// store results
			array_push($galaxyview_data, $system_data);
		}
		unset($galaxyviews_items);
		unset($xdoc);

		// parse data
		$result = $this->update_database($galaxyview_data,$this->userid);
		// add error or success messages
		$this->check_result($result);

		return $result;
	}

	private function get_system_data($galxyview_DOMNode) {
		$return_value = array();
		$return_value["galaxy"] = $galxyview_DOMNode->getAttribute("galaxy");
		$return_value["system"] = $galxyview_DOMNode->getAttribute("system");
		$return_value["data"] = array();

		$position_items = $galxyview_DOMNode->getElementsByTagName("position"); // 15
		foreach($position_items as $position_item) {
			// extract information for each planet position
			$position_data = $this->get_position_data($position_item);
			array_push($return_value["data"],$position_data);
		}
		return $return_value;
	}

	private function get_position_data($position_DOMNode) {
		$return_value = array();

		$return_value["position"] = $position_DOMNode->getAttribute("pos");
		if ($position_DOMNode->hasChildNodes() === false) {
			// no information at this node (not even a debris field)
			return $return_value;
		}

		$activity = $position_DOMNode->getElementsByTagName("activity"); // at max one
		if ($activity->length != 0) {
			$return_value["activity"] = $activity->item(0);
		}

		$return_value["planetname"] = trim($this->get_planetname($position_DOMNode));
		$return_value["moonsize"]   = $this->get_moonsize($position_DOMNode);
		$return_value = array_merge($return_value, $this->get_debris($position_DOMNode));
		$player_data  = $this->get_player_data($position_DOMNode);
		$return_value = array_merge($return_value, $player_data);
		$ally_data    = $this->get_ally_data($position_DOMNode);
		$return_value = array_merge($return_value, $ally_data);


		if ($player_data["playername"] != "") {
			// store alliance name to player name
			$this->collected_playernames[$player_data["playername"]]["allyname"] = $ally_data["allyname"];
		}
		return $return_value;
	}


	private function get_planetname($position_DOMNode) {
		$planetname = $position_DOMNode->getElementsByTagName("planetname"); // at max one
		if ($planetname->length == 0) {
			return "";
		} else {
			return $planetname->item(0)->nodeValue;
		}
	}

	private function get_moonsize($position_DOMNode) {
		$moon = $position_DOMNode->getElementsByTagName("moon"); // at max one
		if ($moon->length == 0) {
			return 0;
		} else {
			return $moon->item(0)->getAttribute("size");
		}
	}

	private function get_debris($position_DOMNode) {
		$return_value = array();

		$debris = $position_DOMNode->getElementsByTagName("debris"); // at max one
		if ($debris->length == 0) {
			$return_value["metal"]   = 0;
			$return_value["crystal"] = 0;
			return $return_value;
		} else {
			// required
			$return_value["metal"]   = $debris->item(0)->getAttribute("metal");
			$return_value["crystal"] = $debris->item(0)->getAttribute("crystal");
			return $return_value;
		}
	}

	private function get_player_data($position_DOMNode) {
		$return_value = array();

		$player = $position_DOMNode->getElementsByTagName("player"); // at max one
		if ($player->length == 0) {
			$return_value["playername"] = "";
			$return_value["rank"]       = 0;
			$return_value["playerid"]   = 0;
			$return_value["status"]     = "";
			return $return_value;
		} else {
			// required
			$return_value["playername"] = trim($player->item(0)->getAttribute("playername"));
			$return_value["rank"]       = $player->item(0)->getAttribute("rank");
			// optional
			if ($player->item(0)->hasAttribute("playerid")) {
				$return_value["playerid"] = $player->item(0)->getAttribute("playerid");
			} else {
				$return_value["playerid"] = 0;
			}
			if ($player->item(0)->hasAttribute("status")) {
				$return_value["status"] = $player->item(0)->getAttribute("status");
			} else {
				$return_value["status"] = "";
			}

			// collect player information
			if (!isset($this->collected_playernames[$return_value["playername"]])) {
				$this->collected_playernames[$return_value["playername"]]["rank"]     = $return_value["rank"];
				$this->collected_playernames[$return_value["playername"]]["playerid"] = $return_value["playerid"];
				$this->collected_playernames[$return_value["playername"]]["status"]   = $return_value["status"];
			}

			return $return_value;
		}
	}

	private function get_ally_data($position_DOMNode) {
		$return_value = array();

		$alliance = $position_DOMNode->getElementsByTagName("alliance"); // at max one
		if ($alliance->length == 0) {
			$return_value["allyname"] = "";
			$return_value["allyid"]   = 0;
			$return_value["rank"]     = 0;
			$return_value["member"]   = "";
			return $return_value;
		} else {
			// required
			$return_value["allyname"] = trim($alliance->item(0)->getAttribute("allyname"));
			$return_value["allyid"]   = $alliance->item(0)->getAttribute("allyid");
			$return_value["rank"]     = $alliance->item(0)->getAttribute("rank");
			$return_value["member"]   = $alliance->item(0)->getAttribute("member");
			// none optional

			// collect alliance information
			if (!isset($this->collected_allynames[$return_value["allyname"]])) {
				$this->collected_allynames[$return_value["allyname"]]["allyid"] = $return_value["allyid"];
				$this->collected_allynames[$return_value["allyname"]]["rank"]   = $return_value["rank"];
				$this->collected_allynames[$return_value["allyname"]]["member"] = $return_value["member"];
			}

			return $return_value;
		}
	}

	private function insert_and_get_allynames($userid) {
		$return_ogame_allynames = array();

		//  Read alliancenames first (in most cases no update should be needed or only very few)
		if (count($this->collected_allynames) == 0) {
			return $return_ogame_allynames;
		}

		// get existing alliances first
		$ally_id_to_ally = array();
		$ogame_allyids   = array();
		$ogame_allynames = array();
		foreach ($this->collected_allynames as $allyname => $allydetails) {
			array_push($ogame_allynames,mysql_real_escape_string($allyname));
			if ($allydetails["allyid"] > 0) {
				// just to be sure

				array_push($ogame_allyids,intval($allydetails["allyid"]));
				$ally_id_to_ally[intval($allydetails["allyid"])] = $allyname;
			}
		}
		// to be backward compatible to old galaxytool entries without alliance ID request also for allyname
		$query = "SELECT distinct(id),allyname,diplomatic_status, ogame_allyid FROM $this->allytable WHERE allyname IN ('".implode("','", $ogame_allynames)."')
				OR ogame_allyid IN('".implode("','", $ogame_allyids)."') AND ogame_allyid > 0";
		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while selecting alliances by name");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}
		unset($ogame_allyids);
		unset($ogame_allynames);

		// check if update on allyname is needed
		while ($line = mysql_fetch_object($res)) {
			if ($line->ogame_allyid > 0 && isset($ally_id_to_ally[$line->ogame_allyid]) && $ally_id_to_ally[$line->ogame_allyid] != $line->allyname) {
				// update on allyname needed (rename will not happen too often - single update is okay)
				$update_allyname = "UPDATE $this->allytable SET allyname='".mysql_real_escape_string($ally_id_to_ally[$line->ogame_allyid])."' WHERE id=$line->id";
				$res2 = $this->run_mysql_query($update_allyname);
				if (!$res2) {
					$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while updating alliance name");
					$this->error_object->add_child_message($this->get_last_mysql_error());
					return false;
				}

				// fill output table with new content
				$return_ogame_allynames[$ally_id_to_ally[$line->ogame_allyid]]["id"]                = $line->id;
				$return_ogame_allynames[$ally_id_to_ally[$line->ogame_allyid]]["diplomatic_status"] = $line->diplomatic_status;
			} else {
				// no update needed - fill output table
				$return_ogame_allynames[$line->allyname]["id"]                = $line->id;
				$return_ogame_allynames[$line->allyname]["diplomatic_status"] = $line->diplomatic_status;
			}
		}

		// get remaining alliance names
		$ally_not_at_db = array();
		foreach ($this->collected_allynames as $allyname => $allydetails) {
			if (!isset($return_ogame_allynames[$allyname])) {
				$ally_not_at_db[$allyname] = $allydetails["allyid"];
			}
		}

		if (count($ally_not_at_db) == 0) {
			return $return_ogame_allynames;
		}

		// insert all those alliances
		$allyname_not_at_db = array();
		$allyinserts = "INSERT INTO $this->allytable (allyname, ogame_allyid, last_update, user_id) VALUES ";
		foreach ($ally_not_at_db as $allyname => $allyid) {
			if ($allyid > 0) {
				$allyinserts .= "('$allyname',$allyid,NOW(),$userid),";
			} else {
				$allyinserts .= "('$allyname',NULL,NOW(),$userid),";
			}
			array_push($allyname_not_at_db,mysql_real_escape_string($allyname));
		}
		// remove last comma
		$allyinserts = substr($allyinserts,0,strlen($allyinserts)-1);
		$res = $this->run_mysql_query($allyinserts);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting alliances");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}

		// determine new IDs of that allynames
		$query = "SELECT distinct(id),allyname,diplomatic_status FROM $this->allytable WHERE allyname IN ('".implode("','", $allyname_not_at_db)."')";
		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while determining alliance IDs");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}

		while ($line = mysql_fetch_object($res)) {
			$return_ogame_allynames[$line->allyname]["id"]                = $line->id;
			$return_ogame_allynames[$line->allyname]["diplomatic_status"] = $line->diplomatic_status;
		}

		return $return_ogame_allynames;
	}

	private function construct_players_query($playername,$playerattributes,$alliance_data,$userid,$with_playerid=false) {
		$query = "(";
		$query .= "'".mysql_real_escape_string($playername,$this->mysql_connection)."',";  		// playername
		if ($with_playerid === true) {
			if ($playerattributes["playerid"] > 0) {											// ogame playerid
				$query .= "'".intval($playerattributes["playerid"])."',";
			} else {
				$query .= "NULL,";
			}
		}

		$query .= "'".intval($playerattributes["rank"])."',";									// rank

		if ($playerattributes["allyname"] != "") {
			$query .= "'".$alliance_data[$playerattributes["allyname"]]["id"]."',"; 	// AllyID
		} else {
			$query .= "0,"; 	// AllyID if no Ally exists
		}

		if (strpos($playerattributes["status"],"n") !== false) {								// noob
			$query .= "'true',";
		} else {
			$query .= "'false',";
		}
		if (strpos($playerattributes["status"],"v") !== false) {								// vacation
			$query .= "'true',";
		} else {
			$query .= "'false',";
		}
		if (strpos($playerattributes["status"],"b") !== false) {								// banned
			$query .= "'true',";
		} else {
			$query .= "'false',";
		}
		if (strpos($playerattributes["status"],"i") !== false) {								// inactive
			$query .= "'true',";
		} else {
			$query .= "'false',";
		}
		if (strpos($playerattributes["status"],"I") !== false) {								// long inactive
			$query .= "'true',";
		} else {
			$query .= "'false',";
		}

		if ($playerattributes["allyname"] != "") {
			$query .= "'".$alliance_data[$playerattributes["allyname"]]["diplomatic_status"]."'"; 	// Ally diplomatic status
		} else {
			$query .= "'nothing'"; 	// default Ally diplomatic status (no alliance)
		}
		$query .= ",$userid),";

		return $query;
	}


	private function construct_players_updatequery($playername,$playerattributes,$alliance_data,$update_playerid=false) {
		$query = "playername='".mysql_real_escape_string($playername,$this->mysql_connection)."',";  	// playername
		$query .= "rank='".intval($playerattributes["rank"])."',";										// rank
		if ($playerattributes["playerid"] > 0 && $update_playerid === true) {
			$query .= "ogame_playerid='".intval($playerattributes["playerid"])."',";					// ogame playerid
		}

		if ($playerattributes["allyname"] != "") {
			$query .= "alliance_id='".$alliance_data[$playerattributes["allyname"]]["id"]."',"; 	// AllyID
		} else {
			$query .= "alliance_id=NULL,"; 	// AllyID
		}

		if (strpos($playerattributes["status"],"n")  !== false) {								// noob
			$query .= "noob='true',";
		} else {
			$query .= "noob='false',";
		}
		if (strpos($playerattributes["status"],"v")  !== false) {								// vacation
			$query .= "vacation='true',";
		} else {
			$query .= "vacation='false',";
		}
		if (strpos($playerattributes["status"],"b")  !== false) {								// banned
			$query .= "banned='true',";
		} else {
			$query .= "banned='false',";
		}
		if (strpos($playerattributes["status"],"i")  !== false) {								// inactive
			$query .= "inactive='true',";
		} else {
			$query .= "inactive='false',";
		}
		if (strpos($playerattributes["status"],"I")  !== false) {								// long inactive
			$query .= "long_inactive='true',";
		} else {
			$query .= "long_inactive='false',";
		}

		if ($playerattributes["allyname"] != "") {
			$query .= "diplomatic_status='".$alliance_data[$playerattributes["allyname"]]["diplomatic_status"]."'"; 	// Ally diplomatic status
		} else {
			$query .= "diplomatic_status='nothing'"; 	// Ally diplomatic status
		}

		return $query;
	}

	private function check_if_playerupdate_required($playertable_object, $playername, $playerattributes, $alliance_data) {
		if ($playertable_object->rank != $playerattributes["rank"]) {
			return true;
		}
		if (isset($alliance_data[$playerattributes["allyname"]]["id"]) &&
		$playertable_object->alliance_id != $alliance_data[$playerattributes["allyname"]]["id"]) {
			// player changed alliance
			return true;
		}
		if (!isset($alliance_data[$playerattributes["allyname"]]["id"]) &&
		$playertable_object->alliance_id != "") {
			// player left alliance
			return true;
		}
		if ($playertable_object->playername != $playername) {
			return true;
		}
		if ($playertable_object->ogame_playerid != $playerattributes["playerid"] && $playerattributes["playerid"] > 0) {
			return true;
		}
		if ($playertable_object->noob == 'true' && (strpos($playerattributes["status"],"n") === false)) {
			return true;
		}
		if ($playertable_object->vacation == 'true' && (strpos($playerattributes["status"],"v") === false)) {
			return true;
		}
		if ($playertable_object->banned == 'true' && (strpos($playerattributes["status"],"b") === false)) {
			return true;
		}
		if ($playertable_object->inactive == 'true' && (strpos($playerattributes["status"],"i") === false)) {
			return true;
		}
		if ($playertable_object->long_inactive == 'true' && (strpos($playerattributes["status"],"I") === false)) {
			return true;
		}

		if ($playertable_object->noob == 'false' && (strpos($playerattributes["status"],"n") !== false)) {
			return true;
		}
		if ($playertable_object->vacation == 'false' && (strpos($playerattributes["status"],"v") !== false)) {
			return true;
		}
		if ($playertable_object->banned == 'false' && (strpos($playerattributes["status"],"b") !== false)) {
			return true;
		}
		if ($playertable_object->inactive == 'false' && (strpos($playerattributes["status"],"i") !== false)) {
			return true;
		}
		if ($playertable_object->long_inactive == 'false' && (strpos($playerattributes["status"],"I") !== false)) {
			return true;
		}
		return false;
	}


	/**
	 * TODO: add more conflict handlings to merge two players without playerid - or wait until ogame provides playerids in all cases
	 *
	 * @param String $playername
	 * @param array $player_attributes
	 * @param array $alliance_data
	 * @return array
	 */
	private function insert_and_get_own_player($playername, $player_attributes, $alliance_data, $userid) {
		$return_ogame_playernames = array();

		// get data for own player (if it exists on DB level already)
		if ($playername != "") {
			if ($this->own_ogame_playerid > 0) {
				$query = "SELECT id,playername,ogame_playerid,rank,alliance_id,noob,vacation,banned,inactive,long_inactive,diplomatic_status
					      FROM $this->playertable WHERE ogame_playerid='".mysql_real_escape_string($this->own_ogame_playerid)."'";
				$res = $this->run_mysql_query($query);
			} else {
				$query = "SELECT id,playername,ogame_playerid,rank,alliance_id,noob,vacation,banned,inactive,long_inactive,diplomatic_status
					      FROM $this->playertable WHERE playername='".mysql_real_escape_string($playername)."'";
				$res = $this->run_mysql_query($query);
			}
			if (!$res) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while reading own player");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
			if (mysql_num_rows($res) > 0) {
				$line = mysql_fetch_object($res);
				// fill returning parameter
				$return_ogame_playernames[$line->playername] = $line->id;

				// update required?
				if ($this->check_if_playerupdate_required($line,$line->playername, $player_attributes, $alliance_data)) {
					// update playerdata
					$query = "UPDATE $this->playertable SET ";
					$query .= $this->construct_players_updatequery($playername,$player_attributes,$alliance_data,true);
					$query .= " WHERE id='$line->id'";
					$res = $this->run_mysql_query($query);
					if (!$res) {
						$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while updating own player data");
						$this->error_object->add_child_message($this->get_last_mysql_error());
						return false;
					}
				}

			} else {
				// we do not know a player with that name on DB level (renamed or never inserted)
				// in case the player was renamed, we have no chance to detect this renaming as the ogame_playerid
				// of that player is unknown
				$query = "INSERT INTO $this->playertable (playername, rank, alliance_id, noob,vacation,banned,inactive,long_inactive,diplomatic_status, user_id) VALUES ";
				$query .= $this->construct_players_query($playername,$player_attributes,$alliance_data,$userid);
				$query = substr($query,0,strlen($query)-1);
				$res = $this->run_mysql_query($query);
				if (!$res) {
					$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting player without ogame playerid");
					$this->error_object->add_child_message($this->get_last_mysql_error());
					return false;
				}
				$id = mysql_insert_id($this->mysql_connection);
				if (is_numeric($id) && $id > 0) {
					// fill returning parameter
					$return_ogame_playernames[$playername] = $id;

				} else {
					// fallback in case mysql_insert_id does not work as expected
					$query = "SELECT id,playername,ogame_playerid,rank,alliance_id,noob,vacation,banned,inactive,long_inactive,diplomatic_status
						      FROM $this->playertable WHERE playername='".mysql_real_escape_string($playername)."'";
					$res = $this->run_mysql_query($query);
					if (!$res) {
						$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while reading previously added player entry");
						$this->error_object->add_child_message($this->get_last_mysql_error());
						return false;
					}
					$line = mysql_fetch_object($res);
					// fill returning parameter
					$return_ogame_playernames[$line->playername] = $line->id;
				}
			}
		}

		return $return_ogame_playernames;
	}

	private function insert_and_get_players($alliance_data, $userid) {
		$return_ogame_playernames = array();
		$ogame_playerids          = array();
		$db_ogame_playerdata      = array();
		$player_wo_id             = ""; // at max one (player itself)
		$player_wo_id_attributes  = "";

		if (count($this->collected_playernames) == 0) {
			// nothing to do
			return $return_ogame_playernames;
		}

		foreach($this->collected_playernames as $playername => $playerattributes) {
			if ($playerattributes["playerid"] > 0) {
				array_push($ogame_playerids,intval($playerattributes["playerid"]));
			} elseif ($player_wo_id == "") { // insert own player only once
				$player_wo_id            = $playername;
				$player_wo_id_attributes = $playerattributes;
			}
		}

		// get data for own player (if it exists on DB level already)
		if ($player_wo_id != "") {
			$return_ogame_playernames = $this->insert_and_get_own_player($player_wo_id, $player_wo_id_attributes, $alliance_data, $userid);
			if (!$return_ogame_playernames) {
				// errors occured
				return $return_ogame_playernames;
			}
		} // own player handling is done now
		unset($player_wo_id);
		unset($player_wo_id_attributes);

		// get data for ogame playerids (if they exists on DB level already)
		if (count($ogame_playerids) == 0) {
			// nothing to do
			return $return_ogame_playernames;
		}

		$query = "SELECT id,playername,ogame_playerid,rank,alliance_id,noob,vacation,banned,inactive,long_inactive,diplomatic_status
				          FROM $this->playertable WHERE ogame_playerid IN ('".implode("','",$ogame_playerids)."')";
		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while reading players by ogame player id");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}
		while ($line = mysql_fetch_object($res)) {
			// fill returning parameter
			$return_ogame_playernames[$line->ogame_playerid] = $line->id;

			/*
			$db_ogame_playerdata[$line->ogame_playerid]["id"]                = $line->id;
			$db_ogame_playerdata[$line->ogame_playerid]["playername"]        = $line->playername;
			$db_ogame_playerdata[$line->ogame_playerid]["rank"]              = $line->rank;
			$db_ogame_playerdata[$line->ogame_playerid]["alliance_id"]       = $line->alliance_id;
			$db_ogame_playerdata[$line->ogame_playerid]["noob"]              = $line->noob;
			$db_ogame_playerdata[$line->ogame_playerid]["vacation"]          = $line->vacation;
			$db_ogame_playerdata[$line->ogame_playerid]["banned"]            = $line->banned;
			$db_ogame_playerdata[$line->ogame_playerid]["inactive"]          = $line->inactive;
			$db_ogame_playerdata[$line->ogame_playerid]["long_inactive"]     = $line->long_inactive;
			$db_ogame_playerdata[$line->ogame_playerid]["diplomatic_status"] = $line->diplomatic_status;
			*/

			$db_ogame_playerdata[$line->ogame_playerid] = $line; // store complete object
		}

		// check if existing entries need an update and collect remaining entries
		$remaining_players     = array();
		$remaining_playernames = array();
		$update_required       = false;
		$query = "INSERT INTO $this->playertable (playername, ogame_playerid, rank, alliance_id, noob,vacation,banned,inactive,long_inactive,diplomatic_status, user_id) VALUES ";
		foreach($this->collected_playernames as $playername => $playerattributes) {
			if ($playerattributes["playerid"] > 0) {
				if (!isset($db_ogame_playerdata[$playerattributes["playerid"]])) {
					$remaining_players[$playername] = $playerattributes;
					array_push($remaining_playernames,mysql_real_escape_string($playername));
				} else {

					if ($this->check_if_playerupdate_required($db_ogame_playerdata[$playerattributes["playerid"]], $playername, $playerattributes, $alliance_data)) {
						$update_required = true;
						$query .= $this->construct_players_query($playername,$playerattributes,$alliance_data,$userid,true);
					}

				}
			}
		}
		if ($update_required == true) {
			$query = substr($query,0,strlen($query)-1);
			$query .= " ON DUPLICATE KEY UPDATE playername=VALUES(playername), alliance_id=VALUES(alliance_id), noob=VALUES(noob), vacation=VALUES(vacation), banned=VALUES(banned), inactive=VALUES(inactive), long_inactive=VALUES(long_inactive) ";
			$res = $this->run_mysql_query($query);
			if (!$res) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting and updating players");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
		}

		// check for remaining player entries which need to be inserted or updated
		// players from other tool users might be in that list as long as they do not have
		// an own ogame_playerid in the table - select with playernames first for remaining
		// entries
		// we might also have this scenario with no solution so far:
		// - Player "A" without playerid is in the list
		// - Player A renamed himself to B without adding new galaxyviews to the tool
		// - another tool user updates a system of B; B will be added with ogame playerID now -> besides A!


		if (count($remaining_players) == 0) {
			return $return_ogame_playernames;
		}


		$query = "SELECT id,playername,ogame_playerid,rank,alliance_id,noob,vacation,banned,inactive,long_inactive,diplomatic_status
			      FROM $this->playertable WHERE playername IN ('".implode("','",$remaining_playernames)."')";
		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while selecting all players by name");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}
		// only use results which have no ogame playerid! == other galaxytool user
		while ($line = mysql_fetch_object($res)) {
			if ($line->ogame_playerid > 0) {
				continue;
			}
			// call updates for ogame playerid (should be one or two max per system) - single updates
			$query = "UPDATE $this->playertable SET ";
			$query .= $this->construct_players_updatequery($line->playername, $this->collected_playernames[$line->playername], $alliance_data, true);
			$query .= " WHERE id='$line->id'";
			$res2 = $this->run_mysql_query($query);
			if (!$res2) {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while updating ogame playerid");
				$this->error_object->add_child_message($this->get_last_mysql_error());
				return false;
			}
			if (isset($remaining_players[$line->playername])) {
				unset($remaining_players[$line->playername]);
			}
			// fill returning parameter
			$return_ogame_playernames[$line->playername] = $line->id;
		}
		unset($remaining_playernames); // not needed any more


		if (count($remaining_players) == 0) {
			return $return_ogame_playernames;
		}

		// insert the rest
		$remaining_playerids = array();
		$query = "INSERT INTO $this->playertable (playername, ogame_playerid, rank, alliance_id, noob,vacation,banned,inactive,long_inactive,diplomatic_status,user_id) VALUES ";

		foreach($remaining_players as $playername => $playerattributes) {
			array_push($remaining_playerids,intval($playerattributes["playerid"]));
			$query .= $this->construct_players_query($playername,$playerattributes,$alliance_data,$userid,true);
		}
		$query = substr($query,0,strlen($query)-1);
		// no duplicate key expected
		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting and updating players");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}

		// get IDs of the rest
		$query = "SELECT id,playername,ogame_playerid,rank,alliance_id,noob,vacation,banned,inactive,long_inactive,diplomatic_status
				  FROM $this->playertable WHERE ogame_playerid IN ('".implode("','",$remaining_playerids)."')";
		$res = $this->run_mysql_query($query);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while selecting remaining players");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		}
		while ($line = mysql_fetch_object($res)) {
			// fill returning parameter
			$return_ogame_playernames[$line->ogame_playerid] = $line->id;
		}

		return $return_ogame_playernames;
	}

	private function update_database($content,$userid) {
		// handle all alliances (maybe we had only empty systems or just players without alliance)
		// array with "ogame allyname => galaxytool_allyid"
		$ogame_allynames = $this->insert_and_get_allynames($userid);

		// handle all players (maybe we had only empty systems)
		// array with "ogame_playerid => galaxytool playerid"
		// or "ogame_playername => galaxytool_playerid" in case the ogame_playerid was not provided (own player)
		$ogame_playernames = $this->insert_and_get_players($ogame_allynames, $userid);

		// now insert all systems

		$query_collection = "INSERT INTO $this->galaxytable (galaxy,system,planet,moon,moonsize,planetname,last_update,metal,crystal,player_id,user_id) VALUES ";
		foreach ($content as $system_data) {
			$galaxy = $system_data["galaxy"];
			$system = $system_data["system"];

			foreach ($system_data["data"] as $position) {
				$pos = $position["position"];

				if (isset($position["moonsize"]) && $position["moonsize"] > 0) {
					$moon     = "true";
					$moonsize = $position["moonsize"];
				} else {
					$moon     = "false";
					$moonsize = 0;
				}


				$planetname = (isset($position["planetname"])) ? $position["planetname"] : "";
				$metal      = (isset($position["metal"]))      ? $position["metal"]      : "0";
				$crystal    = (isset($position["crystal"]))    ? $position["crystal"]    : "0";
				$ogame_pid  = (isset($position["playerid"]))   ? $position["playerid"]   : "0";
				$playername = (isset($position["playername"])) ? $position["playername"] : "";

				// insert row into database
				$query_collection .= " (";
				$query_collection .= "'$galaxy', ";
				$query_collection .= "'$system', ";
				$query_collection .= "'$pos', ";

				$query_collection .= "'$moon', "; // moon
				$query_collection .= "'$moonsize', "; // moonsize
				$query_collection .= "'$planetname', "; // planetname
				$query_collection .= "NOW(), "; // last update
				$query_collection .= "'$metal', "; // metal
				$query_collection .= "'$crystal', "; // crystal

				if (empty($planetname)) {
					$query_collection .= "'0',"; // no player at all
				} elseif (isset($ogame_playernames[$ogame_pid])) {
					$query_collection .= "'".$ogame_playernames[$ogame_pid]."', "; // playerid by ogame_playerid
					// inform activity parser
					if (isset($position['activity']) && $position['activity'] != null) {
						$this->activity_parser->collect_activity($position['activity'],$ogame_playernames[$ogame_pid], ActivityParser::type_galaxyview);
					}
				} else {
					$query_collection .= "'".$ogame_playernames[$playername]."',";	// playerid by playername
					// inform activity parser
					if (isset($position['activity']) && $position['activity'] != null) {
						$this->activity_parser->collect_activity($position['activity'],$ogame_playernames[$playername], ActivityParser::type_galaxyview);
					}
				}

				$query_collection .= "'".$userid."'"; // store userid of the player who updated the system
				$query_collection .= "), ";
			}
			// we added one system to the query
			//if ($this->debug === true) {
			//	$this->error_object = new ErrorObject(ErrorObject::severity_info , "galaxy: ".$galaxy."  system: ".$system."\n");
			//}
		}

		// remove comma
		$query_collection = substr($query_collection,0,strlen($query_collection)-2);
		$query_collection .= " ON DUPLICATE KEY UPDATE moon=VALUES(moon), moonsize=VALUES(moonsize),planetname=VALUES(planetname), last_update=VALUES(last_update),metal=VALUES(metal), crystal=VALUES(crystal),player_id=VALUES(player_id), user_id=VALUES(user_id)";
		$res = $this->run_mysql_query($query_collection);
		if (!$res) {
			$this->error_object = new ErrorObject(ErrorObject::severity_error , "MySQL error occurred while inserting and updating galaxyview");
			$this->error_object->add_child_message($this->get_last_mysql_error());
			return false;
		} else {
			// update activity table too
			$result = $this->activity_parser->insert_into_database();

			if ($result === true) {
				return true;
			} else {
				$this->error_object = new ErrorObject(ErrorObject::severity_error , "Error while updating player activities");
				$this->error_object->add_child_message($this->activity_parser->get_error_object());
				return false;
			}

		}

	}

}

?>